@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #-*- coding:utf-8 -*-
-#
+
 '''
  handler.py
 
@@ -8,10 +8,11 @@
  2013-11-30[22:32:36]:created
  2013-12-11[10:51:16]:better error report
  2013-12-16[17:16:32]:category of tpls
+ 2013-12-25[15:17:46]:s function supported,showing static resource
 
  @info yinyong,osx-x64,UTF-8,192.168.1.101,py,/Users/yinyong/work/ursa2/src
  @author yanni4night@gmail.com
- @version 0.0.1
+ @version 0.0.2
  @since 0.0.1
 '''
 import utils
@@ -23,20 +24,17 @@ import subprocess
 import re
 import json
 from conf import C,log,BASE_DIR
-from render import TokenRender,render_file#render,render_file,getData,getDepsCss,removeCssDepsDeclaration
+from render import TokenRender,render_file
 from replace import replace
 from timestamp import html_link,html_script,html_img,all_url
 from jinja2 import TemplateNotFound,TemplateSyntaxError
 
-
-mimetypes.init()
-
 def _token(path):
     '''
     去除后缀，取得相对于模板目录的相对无扩展名路径
     '''
     tpl_token = re.sub(r'\.(%s|%s|%s|%s)$'%(C('preview_ext'),'m','data',C('template_ext')),'',path)
-    tpl_token = utils.filterRelPath(tpl_token)#re.sub(r'^/*','',tpl_token)
+    tpl_token = utils.filterRelPath(tpl_token)
     return tpl_token
 
 
@@ -53,18 +51,47 @@ def index(req,res):
 
     for e in tpls:
         e = re.sub(r'\.%s'%tpl_ext,'',e)
+        if C('ignore_parents') and e.endswith('parent'):
+            continue
         #分类输出
-        if e.startswith(C('module_dir')):
+        if e.startswith(C('module_dir') + '/'):
             _module_tpls.append(e)
-        elif e.startswith(C('common_dir')):
+        elif e.startswith(C('common_dir') + '/'):
             _common_tpls.append(e)
         else:
             _tpls.append(e);
 
-    index_path = os.path.join(BASE_DIR,'tpl','index.html')
+    index_path ='index.html'
     html = render_file(index_path,{"tpls":_tpls,"module_tpls":_module_tpls,"common_tpls":_common_tpls},noEnvironment = True)
     res.send(html)
 
+def help(req,res):
+    '''
+    加载帮助文件
+    '''
+    inner_path = 'help.html'
+    content =utils.readfile( os.path.join(BASE_DIR,"../","README.md") )
+    html = render_file(inner_path,{'content':content},noEnvironment = True)
+    res.send(html)
+
+def s(req,res):
+    '''
+    静态资源列表
+    '''
+    static_dir = C('static_dir')
+    img_dir = os.path.join(static_dir,C('img_dir'))
+    js_dir = os.path.join(static_dir,C('js_dir'))
+    css_dir = os.path.join(static_dir,C('css_dir'))
+
+    imgs = utils.FileSearcher(r'\.(png|bmp|gif|jpe?g|ico)$',img_dir).search()
+    csses =  utils.FileSearcher(r'\.(css|less)$',css_dir).search()
+    jses =  utils.FileSearcher(r'\.js$',js_dir).search()
+
+    static_path = 'static.html'#os.path.join(BASE_DIR,'tpl','static.html')
+    html = render_file(static_path,{"img":imgs,"css":csses,"js":jses,"img_dir":img_dir,"css_dir":css_dir,"js_dir":js_dir},noEnvironment = True)
+    res.send(html)
+
+
 def tpl(req,res):
     '''
     模板
@@ -79,21 +106,6 @@ def tpl(req,res):
         return res.send(code = 500,content = 'Template %s:%d Syntax Error:%s' % (e.filename,e.lineno,e.message))
     except Exception, e:
         return res.send(code = 500,content = '%s'%e)
-    # if C('server_add_timestamp'):
-    #     base_dir = os.path.join('.',os.path.dirname(req.path))
-    #     html = html_script(html,base_dir)
-    #     html = html_link(html,base_dir)
-    #     html = html_img(html,base_dir)
-    #     html = all_url(html,base_dir)
-    # html = replace(html)
-
-    # #嗅探是否有HTML root标签，如果没有，认为是子模板，需要添加parent支持
-    # if not re.match(r'<html[\s\S]+<body',html,re.I):
-    #     css_deps = getDepsCss(html)
-    #     subparent = os.path.join(BASE_DIR,"tpl",'subparent.tpl')
-    #     html = render_file(subparent,{'name': tpl_token,'content': html,'required_css': css_deps},noEnvironment = True)
-
-    # res.send(html)
 
 def data(req,res):
     '''
@@ -101,7 +113,7 @@ def data(req,res):
     '''
     tpl_token = _token(req.path)
     tr = TokenRender(tpl_token)
-    data = tr.getData()#getData(tpl_token)
+    data = tr.getData()
     return res.send(json.dumps(data),headers = {'Content-Type':'application/json'})
 
 def m(req,res):
@@ -115,7 +127,7 @@ def m(req,res):
         data = utils.readfile(json_path)
     except Exception, e:
         log.error('[m]%s:%s'%(json_path,e))
-    mgr_path = os.path.join(BASE_DIR,'tpl','mgr.html')
+    mgr_path = 'mgr.html'# os.path.join(BASE_DIR,'tpl','mgr.html')
     html = render_file(mgr_path,{"name":tpl_token,"data":data},noEnvironment = True)
     res.send(html)
 
@@ -132,7 +144,7 @@ def so(req,res):
         utils.writeJSON(json_path,json_obj)
         res.redirect('/%s.%s'%(tpl_token,C('preview_ext')))
     except ValueError, e:
-        res.send('<html><head><meta charset = "'+C('encoding')+'"/><title>JSON Error</title></head><body><pre>%s</pre><br/>is not a JSON,<a href = "%s.m">rewrite</a></body></html>'%(json_str,tpl_token))
+        res.send(render_file('json.html',{'json' : json_str,'tpl_token' : tpl_token} , noEnvironment = True))
     except Exception,e:
         log.error('[so]%s'%e)
         res.redirect('/%s.%s'%(tpl_token,C('preview_ext')))
@@ -142,7 +154,7 @@ def static(req,res):
     '''
     static resource
     '''
-    req.path = utils.filterRelPath(req.path)#re.sub(r'/{2,}','/',req.path)
+    req.path = utils.filterRelPath(req.path)
     o = urlparse(req.path)
     #取得绝对路径
     fd = utils.abspath(o.path)
@@ -160,7 +172,7 @@ def static(req,res):
         try:
             headers = {}
             #todo  custom defined
-            if not re.match(utils.BINARY_CONTENT_TYPE_KEYWORDS,content_type,re.IGNORECASE):
+            if not utils.isBinary(fd):
                 content = utils.readfile(fd)
                 content = replace(content)
                
@@ -184,7 +196,7 @@ def static(req,res):
             res.send(content = content,headers = headers)
         except Exception, e:
             res.send(code = 500,content = '%s'%e)
-            log.error(e)
+            log.error("[static]%s"%e)
     else:
         if os.path.exists(fd):
             res.send(code = 403,content = 'Access %s is forbidden'%req.path)
